/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// datastruct.h - definition of data structures used by the wxn suite
//

#ifndef _DATASTRUCT_H_
#define _DATASTRUCT_H_

#include <time.h>
#include <libpq-fe.h>

#include "common.h"

struct type2
{
    int val;
    time_t dtg;
};

struct type3
{
    int peak;
    int duration;   // in seconds
    int trip_point;
    time_t dtg;
};


// wind structure for speed and direction.

struct wind
{
    int spd;
    int dir;
};


// wind structure for speed, direction, and when.

struct wind2
{
    int spd;
    int dir;
    time_t dtg;
};

////////////////////////////////////////////////

struct datatime
{
    time_t tAData;
    time_t tBData;
    time_t tCData;
    time_t tDData;
    time_t tEData;
};

#define DATATIME_SIZE sizeof( struct datatime )

////////////////////////////////////////////////
// node data structure

struct nodedata
{
    int tcr;            // temp - current
    type2 thr;          // temp high (value, time)
    type2 tlr;          // temp low (value, time)
    int trd;            // temp rate - 24 hr
    int trh;            // temp rate - 1 hr
    int tsd;            // soil temp at 6" depth
    int tsd2;           // soil temp at 8" depth
    int tyh;            // yesterday's high
    int tyl;            // yesterday's low

    wind wac;           // current wind average (dir, speed)
    wind wgc;           // current wind gust (dir, speed)
    wind2 wah;          // wind gust average high (dir, speed, time)
    wind2 wal;          // wind gust average low (dir, speed, time)
    wind2 wgh;          // wind gust high (dir, speed, time)

    int wca;            // wind chill average (deg F.)
    int wcg;            // wind chill gust (deg F.)

    int pca;            // rain since midnight local time
    int pri;            // rain rate - instantaneous
    int prh;            // rain rate - hour
    int plh;            // rain last hour
    int p24;            // rain last 24 hours
    int pty;            // rain total yesterday
    int ptm;            // rain total month
    int pta;            // rain total annual

    int bcr;            // barometer - current
    struct type2 bhr;   // barometer high (value, time)
    struct type2 blr;   // barometer low (value, time)
    int brd;            // barometer rate - 24 hr
    int brh;            // barometer rate - 1 hr

    int hcr;            // humidity - current
    struct type2 hhr;   // humidity high (value, time)
    struct type2 hlr;   // humidity low (value, time)
    int hrd;            // humidity rate - 24 hr
    int hrh;            // humidity rate - 1 hr

    // inside temp, humidity
    int ist;            // inside temp
    int ish;            // inside humidity

    // emp
    unsigned ecc;       // current (5 min)
    unsigned eqc;       // 15 min
    unsigned exc;       // 30 min
    unsigned ehc;       // hour
    unsigned edc;       // day (cumulative)

    // background radiation counts (n for nuke...)
    int rca;            // current (5 min) avg
    int rha;            // hour avg
    int rda;            // day avg
    int rmax;           // day peak
    int ralarm_count;   // readings above alarm trip point

    // UV
    int uvc;
    struct type2 uvh;
    struct type2 uvl;

    // sunshine duration for 24 hr period in minutes
    BOOL bSunNow;
    int nSunMin;

    // calculated fields

    BOOL bFog;

    int nHeatIndex;
    int nDewPoint;

    int nHeatDays;
    int nCoolDays;

    time_t tSunrise;
    time_t tSunset;

    BOOL bNwsAlert;

};

#define NODEDATA_SIZE sizeof( struct nodedata )

////////////////////////////////////////////////

struct noderef
{
    char szCallSsid[CALLSSID_SIZE+1];   // this field will be the primary key for db
    char szAlias[CALLSSID_SIZE+1];      // if no alias, same as call-ssid
    char szCity[CITY_SIZE+1];
    char szCounty[COUNTY_SIZE+1];
    char szState[STATE_SIZE+1];
    char szFips[FIPS_SIZE+1];
    char szAfos[AFOS_SIZE+1];
    int nStnType;
    double dLatitude;
    double dLongitude;
    int nTimeZone;          // time zone
    int nElevation;
    int nHAAT;              // anemometer HAAT
    int nAprSoftType;
    BOOL bDst;              // dst flag
    time_t tUpdate;         // time of last update
    int nRadAlarmTrip;
};

#define NODEREF_SIZE sizeof( struct noderef )

////////////////////////////////////////////////

struct nodeblk
{
    struct noderef Info;
    struct nodedata Data;
};

#define NODEBLK_SIZE sizeof( struct nodeblk )


struct noderx
{
    noderef Info;
    nodedata Data;
    datatime Recv;
};

#define NODERX_SIZE sizeof( struct noderx )


struct ini
{
    char szParam[32];
    int nType;
    void * pType;
};


#endif  // _DATASTRUCT_H_

