/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// fips.cpp - FIPS and geographic database interface
//

#define _FIPS_CPP_

#include <string.h>
#include <stdlib.h>
#include <libpq-fe.h>

#include "fips.h"


/////////////////////////////////////////////////////////////////////
// SetFips - set FIPS string argument to FIPS code for a given city,
// county and state.  Returns TRUE success, FALSE otherwise.
//

BOOL SetFips( char * szState, char * szCounty, char * szCity, char * szFips )
{

    PGconn * pConn;
    PGresult * pResult;
    char szExec[384];
    BOOL bStatus;

    pConn = PQconnectdb( "dbname=wxn" );
    if( PQstatus( pConn ) != CONNECTION_OK )
        return FALSE;

    bStatus = FALSE;

    sprintf( szExec, "SELECT fips FROM gdb WHERE state='%s' AND county='%s' AND city='%s'",
        szState, szCounty, szCity );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        strcpy( szFips, PQgetvalue( pResult, 0, 0 ) );
        bStatus = TRUE;
    }
    PQclear( pResult );
    PQfinish( pConn );

    return bStatus;
}


/////////////////////////////////////////////////////////////////////////////
// GetGeoData - searches geographic database for matching city and FIPS code.
// If found, fills variables with data and returns TRUE. Otherwise returns
// FALSE.
//

BOOL GetGeoData( struct noderx * pBlock )
{
    char szExec[384];
    double dLongitude;
    double dLatitude;
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( PQstatus( pConn ) != CONNECTION_OK )
        return FALSE;

    sprintf( szExec, "SELECT county, state, latitude, longitude, afos, zone, elev, dst "
        "FROM gdb WHERE fips='%s' AND city='%s'",
        pBlock->Info.szFips, pBlock->Info.szCity );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        strcpy( pBlock->Info.szCounty, PQgetvalue( pResult, 0, 0 ) );
        strcpy( pBlock->Info.szState, PQgetvalue( pResult, 0, 1 ) );
        dLatitude = atof( PQgetvalue( pResult, 0, 2 ) );
        dLongitude = atof( PQgetvalue( pResult, 0, 3 ) );
        strcpy( pBlock->Info.szAfos, PQgetvalue( pResult, 0, 4 ) );
        pBlock->Info.nTimeZone = atoi( PQgetvalue( pResult, 0, 5 ) );

        if( PQgetisnull( pResult, 0, 6 ) )
            pBlock->Info.nElevation = 0;
        else
            pBlock->Info.nElevation = atoi( PQgetvalue( pResult, 0, 6 ) );

        if(  *PQgetvalue( pResult, 0, 7 ) == 't' )
            pBlock->Info.bDst = TRUE;
        else
            pBlock->Info.bDst = FALSE;

        if( (pBlock->Info.dLatitude == 0) && (pBlock->Info.dLongitude == 0) )
        {
            pBlock->Info.dLatitude = dLatitude;
            pBlock->Info.dLongitude = dLongitude;
        }
    }

    PQclear( pResult );
    PQfinish( pConn );

    return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// GetFips - searches in the geographic database for matching city and state
// If found, fills 'szFips' variable with data and returns TRUE. Otherwise
// returns FALSE.
//

//BOOL GetFips( char * szCity, char * szCounty, char * szState, char * szFips )
BOOL GetFips( noderef * pInfo )
{

    char szExec[384];
    BOOL bStatus;
    PGconn * pConn;
    PGresult * pResult;

    pConn = PQconnectdb( "dbname=wxn" );
    if( PQstatus( pConn ) != CONNECTION_OK )
        return FALSE;

    if( *pInfo->szCounty )
    {
        sprintf( szExec, "SELECT fips FROM gdb WHERE city='%s' AND county='%s' AND state='%s'",
            pInfo->szCity, pInfo->szCounty, pInfo->szState );
    }
    else
    {
        sprintf( szExec, "SELECT fips FROM gdb WHERE city='%s' AND state='%s'",
            pInfo->szCity, pInfo->szState );
    }
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        // until an alternative is devised, will return first record found if multiple records
        strcpy( pInfo->szFips, PQgetvalue( pResult, 0, 0 ) );
        bStatus = TRUE;
    }
    else
        bStatus = FALSE;

    PQclear( pResult );
    PQfinish( pConn );

    return bStatus;

}

