/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//////////////////////////////////////////////////////////////////////////////
// aconfig.cpp
//
//                           -------------------
//  begin                : Wed Feb  5 11:47:20 CST 2003
//  copyright            : (C) 2003 by John Bennett
//  email                : n4xi@arrl.net
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
////////////////////////////////////////////////////////////////////////////

#define _ACONFIG_CPP_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

//libwxn
#include <logmsg.h>

// other
#include "aconfig.h"
#include "aglobal.h"

// init names, types, variables

#define STRING  0
#define CHAR    1
#define BOOLEAN 2
#define INTEGER 3
#define SHORT   4
#define UNS_INT 5
#define TIME    6
#define LONG    7
#define MULTISZ 8
#define FLINT   9
#define FLOAT   10

aprconf g_AprConfig;

struct ini AprIni[] =
{
    { "UseQrz",             BOOLEAN,    &g_AprConfig.bUseQrz },
    { "QrzPath",            STRING,     &g_AprConfig.szQrzPath },
    { "SendAlarmData",      BOOLEAN,    &g_AprConfig.bSendAlarmData },
    { "AprsExtraText",      STRING,     &g_AprConfig.szAprsText },
    { "AprsExtraInterval",  INTEGER,    &g_AprConfig.nAprsExtraInterval },
    { "AprsDelayThreshold", INTEGER,    &g_AprConfig.nAprsDelayThreshold },
    { "AprsReceive",        BOOLEAN,    &g_AprConfig.bAprsReceive },
    { "ReceiveLocal",       BOOLEAN,    &g_AprConfig.bRcvLocal },
    { "UseCallbook",        BOOLEAN,    &g_AprConfig.bUseCallbook }
};

#define APR_INI_ENTRIES (sizeof(AprIni)/ sizeof(struct ini))



/////////////////////////////////////////////////////////////////////////////
// ReadAprConfig - read <wxnaprd.conf> to set required parameters.
// Returns TRUE success, FALSE failure.
//

BOOL ReadAprConfig( void )
{
    FILE * hFile;
    BOOL bFound;
    BOOL bNoMatch;
    int i;
    char szString[STRING_SIZE];
    char szLine[STRING_SIZE];
    char szMsg[STRING_SIZE];
    char * pS;
    char * pParam;
    char * pVal;
    double dVal;

    hFile = fopen( WXN_SYSCONFDIR "wxnaprd.conf", "r" );
    if( hFile == NULL )
    {
        g_log.PostMsg( APRMSG_WXNAPRD_CONF_FAIL, "cannot open/read file" );
        return FALSE;
    }

    // set defaults

    memset( &g_AprConfig, 0, sizeof(struct aprconf) );

    g_AprConfig.nAprsExtraInterval = 20;
    g_AprConfig.nAprsDelayThreshold = 0;
    g_AprConfig.bAprsReceive = TRUE;

    FOREVER     // for(;;)
    {
        if( fgets( szLine, STRING_SIZE, hFile ) == NULL )
            break;      // EOF

        if( *szLine == '\r' || *szLine == '\n' )
            continue;               // empty line

        pS =strchr( szLine, 0 );
        if( *(pS-1) == '\r' || *(pS-1) == '\n' )
            *(pS-1) = 0;

        strcpy( szString, szLine );

        bFound = FALSE;
        bNoMatch = TRUE;

        pParam = szString;
        while( *pParam )
        {
            if( *pParam != ' ' )
                break;
            pParam++;
        }

        if( *pParam == '#' )      // comment
            continue;

        pVal = pParam;
        while( *pVal )
        {
            if( *pVal == ' ' ) {
                *pVal++ = 0;        // terminate the parameter portion of string
                continue;
                }
            if( *pVal++ == '=' )
                break;
        }
        while( *pVal )
        {
            if( *pVal != ' ' )
                break;
            pVal++;
        }

        if( *pVal == '"' )          // check for enclosing quotes
            pVal++;
        if( strchr( pVal, '"' ) )
            *strchr( pVal, '"' ) = 0;

        if( *pVal == 0 )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( APRMSG_WXNAPRD_CONF_FAIL, szMsg );
            continue;
        }

        for( i=0; i<(int)APR_INI_ENTRIES; i++ )
        {
            if( strcasecmp( pParam, AprIni[i].szParam ) == 0 )
            {
                bFound = TRUE;
                bNoMatch = FALSE;
                if( pVal == NULL )
                    break;
                switch( AprIni[i].nType )
                    {
                    case STRING:
                        strcpy( (char *) AprIni[i].pType, pVal );
                        break;

                    case CHAR:
                        *(char *) AprIni[i].pType = *pVal;
                        break;

                    case BOOLEAN:
                        if( ((*pVal & 0x005f) == 'T') || ((*pVal & 0x005f) == 'Y') )
                            *((BOOL *) AprIni[i].pType) = TRUE;
                        else
                            *((BOOL *) AprIni[i].pType) = FALSE;
                        break;

                    case INTEGER:
                        *((int *) AprIni[i].pType) = atoi( pVal );
                        break;

                    case SHORT:
                        *((short *) AprIni[i].pType) = (short) atoi( pVal );
                        break;

                    case UNS_INT:
                        *((unsigned *) AprIni[i].pType) = (unsigned) atol( pVal );
                        break;

                    case TIME:
                        *((time_t *) AprIni[i].pType) = (time_t) atol( pVal );
                        break;

                    case LONG:
                        *((long *) AprIni[i].pType) = atol( pVal );
                        break;

                    case FLINT:
                        dVal = atof( pVal ) * 100;
                        *((int *) AprIni[i].pType) = (int) dVal;   // one-liner introduces error
                        break;

                    case FLOAT:
                        dVal = atof( pVal );
                        *((double *) AprIni[i].pType) = dVal;
                        break;

                    default:
                        bNoMatch = TRUE;
                        break;
                }
            }

            if( bFound )
                break;
            }

        if( bNoMatch )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( APRMSG_WXNAPRD_CONF_FAIL, szMsg );
        }
    }

    fclose( hFile );

    if( *g_AprConfig.szQrzPath == 0 )
        strcpy( g_AprConfig.szQrzPath, "/mnt/cdrom/callbk/" );
    else
    {
        pS = strchr( g_AprConfig.szQrzPath, 0 ) - 1;
        if( *pS != '/' )
            strcat( g_AprConfig.szQrzPath, "/" );
    }

    return TRUE;
}

