/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// aprs.cpp - class handler for native APRS functions
//

#define _APRS_CPP_

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>

// libwxn
#include <crc.h>
#include <strxcpy.h>
#include <wxcalc.h>
#include <trim.h>
#include <spacedelimit.h>
#include <tz.h>
#include <logmsg.h>

#include "aprs.h"
#include "dupe.h"
#include "round.h"


//////////////////////////////////////////////////////////////////////////////
// Constructor

Aprs::Aprs()
{
    *m_sTxStnType = 0;
    *m_szRxToCall = 0;
    *m_szRxFmCall = 0;
    *m_szExtraText = 0;
    *m_szTxFmCall = 0;

    m_nDecodeType = NO_TYPE;
    m_nIGateCnt = 0;
    m_nIGateIndex = 0;
}


//////////////////////////////////////////////////////////////////////////////
// Destructor

Aprs::~Aprs()
{

}


//////////////////////////////////////////////////////////////////////////////
// setup

BOOL Aprs::setup( char * pFmCall, char * pToCall, char * pDigis, char * pPort, char * pText, BOOL bRcvLocal )
{
    strcpy( m_TxDigis, pDigis );
    strcpy( m_TxToCall, pToCall );
    extra( pText );

    strcpy( m_szTxFmCall, pFmCall );

    return Ax25::setup( pFmCall, pPort, bRcvLocal );
}


//////////////////////////////////////////////////////////////////////////////
// extra - overloaded functions set/retrieve extra packet text

void Aprs::extra( char * pText )
{
    if( !pText )
        return;

    if( isdigit( *pText ) ) {
        *m_szExtraText = ' ';
        strcpy( m_szExtraText+1, pText );
    }
    else
        strcpy( m_szExtraText, pText );

    strcat( m_szExtraText, "\r" );
}

char * Aprs::extra( void )
{
    return m_szExtraText;
}


//////////////////////////////////////////////////////////////////////////////
// digis - overloaded functions set/retrieve digipeater path

void Aprs::digis( char * pDigis )
{
    strcpy( m_TxDigis, pDigis );
}


char * Aprs::digis( void )
{
    return m_TxDigis;
}


//////////////////////////////////////////////////////////////////////////////
// SetStationType

void Aprs::SetStationType( int nType )
{
    switch( nType )
    {
        case ID5001:
            strcpy( m_sTxStnType, "id5" );
            break;

        case ULTII:
            strcpy( m_sTxStnType, "um2" );
            break;

        case ULT2000:
            strcpy( m_sTxStnType, "u2k" );
            break;

        case DAVIS2:
        case DAVISVP:
            strcpy( m_sTxStnType, "dav" );
            break;

        default:
            *m_sTxStnType = 0;
            break;

    }
}


//////////////////////////////////////////////////////////////////////////////
// SetCoordinates

void Aprs::SetCoordinates( double dLatitude, double dLongitude )
{
    int nDeg;
    int nMin;
    double dTemp1, dTemp2;
    char szTemp[32];
    char cDir;

    dTemp1 = dLatitude;
    nDeg = (int) dTemp1;
    dTemp2 = dTemp1 - (double) nDeg;
    dTemp1 = dTemp2 * 60;

    nMin = (int) dTemp1;
    dTemp2 = dTemp1 - (double) nMin;
    dTemp1 = (double) (nDeg * 100) + (double) nMin +dTemp2;

    if( dTemp1 < 0 )
        cDir = 'S';
    else
        cDir = 'N';

    sprintf( szTemp, "%07.2f%c", dTemp1, cDir );
    strcpy( m_sTxCoordinates, szTemp );

    dTemp1 = dLongitude;
    nDeg = (int) dTemp1;
    dTemp2 = dTemp1 - (double) nDeg;
    dTemp1 = dTemp2 * 60;
    nMin = (int) dTemp1;

    dTemp2 = dTemp1 - (double) nMin;
    dTemp1 = (double) (nDeg * 100) + (double) nMin +dTemp2;

    if( dTemp1 < 0 )
        cDir = 'E';
    else
        cDir = 'W';

    sprintf( szTemp, "/%08.2f%c", dTemp1, cDir );
    strcat( m_sTxCoordinates, szTemp );
}


//////////////////////////////////////////////////////////////////////////////
// beacon

void Aprs::beacon( noderx * pBlock )
{
    struct tm * pTm;
    char szTemp[32];
    int nSpeed;
    int nLen;
    int nDir;

    nSpeed = (int) ( ( (long)(pBlock->Data.wac.spd) * (long)5280 ) / (long)6076 );

    if( pBlock->Data.wac.dir == -1 )
        nDir = 0;
    else
        nDir = pBlock->Data.wac.dir;

    pTm = localtime( &pBlock->Info.tUpdate );

    sprintf( m_sInf, "!%s_%03i/%03ig%03it%03ir%03ip%03iP%03i",
        m_sTxCoordinates,
        nDir, nSpeed, pBlock->Data.wgc.spd,
        pBlock->Data.tcr,
        pBlock->Data.prh, pBlock->Data.p24, pBlock->Data.pca );

    if( pBlock->Data.hcr )
    {
        if( pBlock->Data.hcr > 99 )
            strcpy( szTemp, "h00" );
        else
            sprintf( szTemp, "h%02i", pBlock->Data.hcr );
    }
    else
        strcpy( szTemp, "h.." );
    strcat( m_sInf, szTemp );

    if( pBlock->Data.bcr )
        sprintf( szTemp, "b%05i", pBlock->Data.bcr );
    else
        strcpy( szTemp, "b....." );
    strcat( m_sInf, szTemp );

    if( pBlock->Data.bNwsAlert )
        sprintf( szTemp, "%c!%s\r", APRS_SOFT_TYPE, m_sTxStnType );

    else
        sprintf( szTemp, "%c%s\r", APRS_SOFT_TYPE, m_sTxStnType );
    strcat( m_sInf, szTemp );

    nLen = strlen( m_sInf );

    put( m_TxToCall, m_TxDigis, m_sInf, nLen );
}


//////////////////////////////////////////////////////////////////////////////
// BeaconExtra - beacon position and optional text string
//

void Aprs::BeaconExtra( void )
{
    int nLen;

    // e.g., !3904.07N/08637.65W_PHG5490/wb9tlh@arrl.net

    if( *m_szExtraText )
    {
        sprintf( m_sInf, "!%s_%s", m_sTxCoordinates, m_szExtraText );
        nLen = strlen( m_sInf );
        put( m_TxToCall, m_TxDigis, m_sInf, nLen );
    }
    // else don't transmit if incomplete...
}


//////////////////////////////////////////////////////////////////////////////
// BeaconWxn - beacon WXN extended data
//

void Aprs::BeaconWxn( char * pData )
{
    int nLen;

    nLen = strlen( pData );
    put( m_TxToCall, m_TxDigis, pData, nLen );
}


//////////////////////////////////////////////////////////////////////////////
// RxFmCall - returns from call-ssid
//

void Aprs::RxFmCall( char * pS )
{
    strcpy( pS, m_szRxFmCall );
}


//////////////////////////////////////////////////////////////////////////////
// RxToCall - returns to call-ssid
//

void Aprs::RxToCall( char * pS )
{
    strcpy( pS, m_szRxToCall );
}


//////////////////////////////////////////////////////////////////////////////
// rxstatus - returns status of receive buffer: NO_DATA, WX_DATA, POSITION,
// WXN_TYPE
//

int Aprs::rxStatus( void )
{
    char szTemp[16];
    int nLength;
    unsigned uCrc;
    char * pS;
    char * pT;

    if( !get() )
        return NO_DATA;

    if( FrameType() != U_FRM_UI )
        return NO_DATA;

    nLength = data( m_szRxBuffer, AX25_FRAME_MAX_SIZE );
    if( nLength == 0 )
        return NO_DATA;
    m_szRxBuffer[nLength] = 0;  // null terminate the string

    // don't allow self
    ToCall( szTemp );
    sprintf( m_szRxToCall, "%s-%i", szTemp, ToSsid() );
    FmCall( szTemp );
    sprintf( m_szRxFmCall, "%s-%i", szTemp, FmSsid() );
    if( strcasecmp( m_szRxFmCall, m_szTxFmCall ) == 0 )
        return NO_DATA;

    // don't process duplicate packets
    uCrc = Crc( m_szRxBuffer, nLength );
    if( m_dupe.InList( m_szRxFmCall, uCrc ) )
        return NO_DATA;
    else
        m_dupe.Add( m_szRxFmCall, uCrc );

    // assume failure
    m_nRxDataType = INVALID_TYPE;

    // IGATE?
    m_nIGateCnt = 0;
    if( m_szRxBuffer[0] == '}' )
    {
        // point to original APRS packet
        m_pRxBuffer = strchr( m_szRxBuffer, ':' );
        if( m_pRxBuffer )
        {
            *m_pRxBuffer = 0;
            m_pRxBuffer++;
        }
        else    // something wrong
            return NO_DATA;
        // set source to source call-ssid in source path header
        pS = strchr( m_szRxBuffer, '>' );
        if( pS == NULL )
            return NO_DATA;
        *pS = 0;    // terminate so only call-ssid remains
        // copy the source call, since this is a "digi" IGATE style
        strxcpy( m_sIDigi[m_nIGateCnt++], m_szRxFmCall, AX25_CALLSSID_SIZE );
        // now replace with actual originating call-ssid
        strxcpy( m_szRxFmCall, m_szRxBuffer+1, AX25_CALLSSID_SIZE );
        // eliminate any trailing '*' on call-ssid
        pT = strchr( m_szRxFmCall, '*' );
        if( pT )
            *pT = 0;
        if( strchr( m_szRxFmCall, '-' ) == NULL )
            strcat( m_szRxFmCall, "-0" );
        pS++; // point to digipeater string
        if( *pS )
        {
            pT = strtok( pS, "," );
            while( pT )
            {
                strcpy( m_sIDigi[m_nIGateCnt], pT );
                pT = strchr( m_sIDigi[m_nIGateCnt], '*' );
                if( pT )
                    *pT = 0;
                m_nIGateCnt++;
                pT = strtok( NULL, "," );
            }
        }
    }
    else
        m_pRxBuffer = m_szRxBuffer;

    if( *m_pRxBuffer == '_' )
    {
        m_nRxDataType = POSITIONLESS;
        return WX_DATA;
    }

    // timestamp, position
    else if( *m_pRxBuffer == '/' || *m_pRxBuffer == '@' )
    {
        if( isdigit( *(m_pRxBuffer+START_OFFSET+TIMESTAMP_LEN) ) )
        {
            // un-compressed position
            if( *(m_pRxBuffer+START_OFFSET+TIMESTAMP_LEN+UPOS_LENGTH) == '_' )
            {
                if( isdigit( *(m_pRxBuffer+START_OFFSET+TIMESTAMP_LEN+UPOS_LENGTH+1) ) )
                {
                    m_nRxDataType = TIME_UPOS_WX;
                    return WX_DATA;
                }
                else
                {
                    m_nRxDataType = TIME_UPOS;
                    return POSITION;
                }
            }

            else
            {
                m_nRxDataType = TIME_UPOS;
                return POSITION;
            }

        }
        else    // compressed position

        {
            if( *(m_pRxBuffer+START_OFFSET+TIMESTAMP_LEN+CPOS_LENGTH) == '_' )
            {
                m_nRxDataType = TIME_CPOS_WX;
                return WX_DATA;
            }

            else
            {
                m_nRxDataType = TIME_CPOS;
                return POSITION;
            }
        }
    }
    // position only
    else if( *m_pRxBuffer == '!' || *m_pRxBuffer == '=' )
    {
        if( isdigit( *(m_pRxBuffer+1) ) )
        {
            // un-compressed position
            if( *(m_pRxBuffer+START_OFFSET+UPOS_LENGTH) == '_' )
            {
                if( isdigit( *(m_pRxBuffer+START_OFFSET+UPOS_LENGTH+1) ) )
                {
                    m_nRxDataType = UPOS_WX;
                    return WX_DATA;
                }

                else
                {
                    m_nRxDataType = UPOS;
                    return POSITION;
                }
            }

            else
            {
                m_nRxDataType = UPOS;
                return POSITION;
            }
        }
        else    // compressed position
        {
            if( *(m_pRxBuffer+START_OFFSET+CPOS_LENGTH) == '_' )
            {
                m_nRxDataType = CPOS_WX;
                return WX_DATA;
            }
            else
            {
                m_nRxDataType = CPOS;
                return POSITION;
            }
        }
    }
    // wxn extended data packets
    else if( *m_pRxBuffer == '{' )
    {

        if( *(m_pRxBuffer+1) == WXN_USER_ID )
        {
            m_nRxDataType = USR_WXN;
            return WXN_DATA;
        }
        else
            return NO_DATA;
    }

    // nothing matches
    return NO_DATA;
}


//////////////////////////////////////////////////////////////////////////////
// rxData - retrieve and process received weather data. Returns TRUE
// success, FALSE failure
//

BOOL Aprs::rxData( noderx * pBlock, time_t tZulu  )
{
    int nOffset;

    strxcpy( pBlock->Info.szCallSsid, m_szRxFmCall, CALLSSID_SIZE );

    m_bPosition = FALSE;

    switch( m_nRxDataType )
    {
        case POSITIONLESS:
            pBlock->Info.tUpdate = tZulu;   // DecodeTime( tZulu );
            WeatherData( pBlock, START_OFFSET+NOPOS_STAMP_LEN, TRUE );
            break;

        case TIME_UPOS_WX:
        case TIME_CPOS_WX:
            m_bPosition = TRUE;
            pBlock->Info.tUpdate = tZulu;   // DecodeTime( tZulu );

            nOffset = DecodePosition( START_OFFSET+TIMESTAMP_LEN );
            nOffset += DecodeWind( &pBlock->Data, nOffset+START_OFFSET+TIMESTAMP_LEN+SYMBOL_LEN );
            WeatherData( pBlock, nOffset+START_OFFSET+TIMESTAMP_LEN, FALSE );
            break;

        case UPOS_WX:
        case CPOS_WX:
            m_bPosition = TRUE;
            pBlock->Info.tUpdate = tZulu;   // real-time
            nOffset = DecodePosition( START_OFFSET );
            nOffset += DecodeWind( &pBlock->Data, nOffset+START_OFFSET+SYMBOL_LEN );
            WeatherData( pBlock, nOffset+START_OFFSET, FALSE );
            break;


        case USR_WXN:
            return WxnDecode( pBlock );
            break;

        default:
            return FALSE;

    }

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// rxpos - retrieve position data
//

BOOL Aprs::rxPos( position * pPos )
{
    switch( m_nRxDataType )
    {
        case TIME_UPOS:
        case TIME_CPOS:
            DecodePosition( START_OFFSET+TIMESTAMP_LEN );
            break;

        case UPOS:
        case CPOS:
            DecodePosition( START_OFFSET );
            break;

        default:
            return FALSE;
    }

    memcpy( pPos, &m_pos, sizeof( position ) );

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// DecodePosition - decodes compressed or uncompressed lat/lng and fills
// member structure with data. Returns  UPOS_LENGTH or CPOS_LENGTH. Used
// by caller to determine offset to next piece of data.
//

int Aprs::DecodePosition( int nOffset )
{
    char szTemp[32];

    if( isdigit( *(m_pRxBuffer+nOffset) ) )  // un-compressed
    {
        m_bCompressedData = FALSE;

        strxcpy( szTemp, m_pRxBuffer+nOffset, 2 );
        m_pos.dLat = atof( szTemp );
        strxcpy( szTemp, m_pRxBuffer+nOffset+2, 5 );
        m_pos.dLat += atof( szTemp )/60;

        strxcpy( szTemp, m_pRxBuffer+nOffset+9, 3 );
        m_pos.dLng = atof( szTemp );
        strxcpy( szTemp, m_pRxBuffer+nOffset+12, 5 );
        m_pos.dLng += atof( szTemp )/60;

        return UPOS_LENGTH;
    }

    else                    // compressed
    {
        m_bCompressedData = TRUE;

        m_pos.dLat = (*(m_pRxBuffer+nOffset+1)-33) * pow(91,3);
        m_pos.dLat += ( *(m_pRxBuffer+nOffset+2)-33) * pow(91,2);
        m_pos.dLat += ( *(m_pRxBuffer+nOffset+3)-33) * 91;
        m_pos.dLat += *(m_pRxBuffer+nOffset+4) - 33;
        m_pos.dLat = 90 -  m_pos.dLat/380926;

        m_pos.dLng = ( *(m_pRxBuffer+nOffset+5)-33) * pow(91,3);
        m_pos.dLng += ( *(m_pRxBuffer+nOffset+6)-33) * pow(91,2);
        m_pos.dLng += ( *(m_pRxBuffer+nOffset+7)-33) * 91;
        m_pos.dLng += *(m_pRxBuffer+nOffset+8) - 33;
        m_pos.dLng = -180 + m_pos.dLng/190463;
        m_pos.dLng *= -1;

        return CPOS_LENGTH;
    }

}


//////////////////////////////////////////////////////////////////////////////
// Position - must be called only after rxData to be valid. Returns TRUE if
// position was encoded in last position, FALSE otherwise.
//

BOOL Aprs::Position( void )
{
    return m_bPosition;
}


//////////////////////////////////////////////////////////////////////////////
// rxLatitude

double Aprs::rxLatitude( void )

{
    if( m_bPosition )
        return m_pos.dLat;
    else
        return 0;
}


//////////////////////////////////////////////////////////////////////////////
// rxLongitude

double Aprs::rxLongitude( void )
{
    if( m_bPosition )
        return m_pos.dLng;
    else
        return 0;
}


//////////////////////////////////////////////////////////////////////////////
// DecodeTime - returns timestamp from APRS text format as type time_t.
// Requires current UTC as argument.
//

time_t Aprs::DecodeTime( time_t tTime )
{
    tm * pTm;
    char szDupe[8];

    pTm = localtime( &tTime );

    if( *(m_pRxBuffer+START_OFFSET+6) == 'z' )            // UTC
    {
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET, 2 );
        pTm->tm_mday = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+2, 2 );
        pTm->tm_hour = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+4, 2 );
        pTm->tm_min = atoi( szDupe );
        pTm->tm_sec = 0;
    }

    else if( *(m_pRxBuffer+START_OFFSET+6) == 'h' )       // hh:mm:ss
    {
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET, 2 );
        pTm->tm_hour = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+2, 2 );
        pTm->tm_min = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+4, 2 );
        pTm->tm_sec = atoi( szDupe );
    }

    else                            // MM:dd:hh:mm (positionless only)
    {
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET, 2 );
        pTm->tm_mon = atoi( szDupe ) - 1;
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+2, 2 );
        pTm->tm_mday = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+4, 2 );
        pTm->tm_hour = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+6, 2 );
        pTm->tm_min = atoi( szDupe );
        strxcpy( szDupe, m_pRxBuffer+START_OFFSET+8, 2 );
    }

    return mktime( pTm );
}


//////////////////////////////////////////////////////////////////////////////
// WeatherData - fill array with received weather data

void Aprs::WeatherData( noderx * pBlock, int nOffset, BOOL bWind )
{
    char * pS;
    char szTemp[64];
    BOOL bBreak;

    // dump CR, if present
    pS = strchr( m_pRxBuffer, '\r' );
    if( pS )
        *pS = 0;

    pS = m_pRxBuffer + nOffset;

    // the first four fields are fixed in position
    if( bWind )
    {
        // wind direction 'c'
        strxcpy( szTemp, pS+1, WIND_DIR_LEN );

        pBlock->Data.wac.dir = atoi( szTemp );
        pS += WIND_DIR_LEN+1;

        // wind speed 's'
        strxcpy( szTemp, pS+1, WIND_SPD_LEN );
        pBlock->Data.wac.spd = atoi( szTemp );
        pS += WIND_SPD_LEN+1;
    }

    // wind gust 'g'
    strxcpy( szTemp, pS+1, GUST_LEN );
    pBlock->Data.wgc.spd = atoi( szTemp );
    pBlock->Data.wgc.dir = -1;      // indicate "error" since APRS does not support natively
    pS += GUST_LEN+1;

    // temperature 't'
    strxcpy( szTemp, pS+1, TEMP_LEN );
    pBlock->Data.tcr = atoi( szTemp );
    pS += TEMP_LEN+1;

    // the following may or may not be present
    bBreak = FALSE;
    //FOREVER
    for(;;)
    {
        switch( *pS )
        {
            case 'r':
                if( isdigit( *(pS+1) ) )
                {
                    strxcpy( szTemp, pS+1, RAIN_HR_LEN );
                    pBlock->Data.plh = atoi( szTemp );
                }
                pS += RAIN_HR_LEN+1;
                break;

            case 'p':
                if( isdigit( *(pS+1) ) )
                {
                    strxcpy( szTemp, pS+1, RAIN_24_LEN );
                    pBlock->Data.p24 = atoi( szTemp );
                }
                pS += RAIN_24_LEN+1;
                break;

            case 'P':
                if( isdigit( *(pS+1) ) )
                {
                    strxcpy( szTemp, pS+1, RAIN_DAY_LEN );
                    pBlock->Data.pca = atoi( szTemp );
                }
                pS += RAIN_DAY_LEN+1;
                break;

            case 'h':
                if( isdigit( *(pS+1) ) )
                {
                    strxcpy( szTemp, pS+1, HUMID_LEN );
                    pBlock->Data.hcr = atoi( szTemp );
                    if( pBlock->Data.hcr == 0 )
                        pBlock->Data.hcr = 100;
                }
                pS += HUMID_LEN+1;
                break;


            case 'b':
                if( isdigit( *(pS+1) ) )
                {
                    strxcpy( szTemp, pS+1, BAROM_LEN );
                    pBlock->Data.bcr = atoi( szTemp );
                }
                pS += BAROM_LEN+1;
                break;

            default:
                bBreak = TRUE;
                break;
        }

        if( bBreak )
            break;
    }

    switch( tolower(*pS) )
    {
        case 'd':
            // APRSdos
            pBlock->Info.nAprSoftType = APRSDOS;
            break;

        case 'm':
            // MacAPRS

            pBlock->Info.nAprSoftType = MACAPRS;
            break;

        case 'p':
            // pocketAPRS
            pBlock->Info.nAprSoftType = POCKETAPRS;
            break;

        case 's':
            // APRS+SA
            pBlock->Info.nAprSoftType = APRSSA;
            break;

        case 'w':
            // WinAPRS
            pBlock->Info.nAprSoftType = WINAPRS;
            break;

        case 'x':
            // WXN
            pBlock->Info.nAprSoftType = XAPRS;
            break;

        case 'n':
            // WXN
            pBlock->Info.nAprSoftType = WXN;
            break;

        default:
            // unknown software type
            pBlock->Info.nAprSoftType = APRSSOFT;
            break;
    }

    pS++;

    // wx stn type
    if( *pS )
    {
        if( strchr( pS, '!' ) )
        {
            pBlock->Data.bNwsAlert = TRUE;
            pS++;
        }
        else
            pBlock->Data.bNwsAlert = FALSE;

        if( strcasecmp( pS, "dvs" ) == 0 )
            pBlock->Info.nStnType = DAVIS2;
        else if( strcasecmp( pS, "dav" ) == 0 )
            pBlock->Info.nStnType = DAVIS2;
        else if( strcasecmp( pS, "hkt wxstn" ) == 0 )
            pBlock->Info.nStnType = ID5001;
        else if( strcasecmp( pS, "pic" ) == 0 )
            pBlock->Info.nStnType = PICWX;
        else if( strcasecmp( pS, "rsw" ) == 0 )
            pBlock->Info.nStnType = WM918;
        else if( strcasecmp( pS, "u-ii" ) == 0 )
            pBlock->Info.nStnType = ULTII;
        else if( strcasecmp( pS, "u2r" ) == 0 )
            pBlock->Info.nStnType = ULTII;
        else if( strcasecmp( pS, "um2" ) == 0 )
            pBlock->Info.nStnType = ULTII;
        else if( strcasecmp( pS, "u2k" ) == 0 )
            pBlock->Info.nStnType = ULT2000;
        else if( strcasecmp( pS, "u2kr" ) == 0 )
            pBlock->Info.nStnType = ULT2000;
        else if( strcasecmp( pS, "u5" ) == 0 )
            pBlock->Info.nStnType = ULT2000;
        else if( strcasecmp( pS, "upkm" ) == 0 )
            pBlock->Info.nStnType = ULT2000;
        else if( strcasecmp( pS, "id5" ) == 0 )
            pBlock->Info.nStnType = ID5001;
        else if( strcasecmp( pS, "osw" ) == 0 )
            pBlock->Info.nStnType = WM918;

        else
            pBlock->Info.nStnType = UNKNOWN;
    }
    else
        pBlock->Info.nStnType = UNKNOWN;

    if( pBlock->Data.hcr )
    {
        pBlock->Data.nDewPoint = DewPoint( pBlock->Data.tcr, pBlock->Data.hcr );
        if( pBlock->Data.tcr > 67 )
            pBlock->Data.nHeatIndex = HeatIndex( pBlock->Data.tcr, pBlock->Data.hcr );
        else
            pBlock->Data.nHeatIndex = 0;
    }
    else
    {
        pBlock->Data.nDewPoint = 0;
        pBlock->Data.nHeatIndex = 0;
    }

}


//////////////////////////////////////////////////////////////////////////////
// DecodeWind
//

int Aprs::DecodeWind( nodedata * pData, int nOffset )
{
    char szTemp[32];

    if( m_bCompressedData )
    {
        pData->wac.dir = (*(m_pRxBuffer+nOffset)-33) * 4;
        pData->wac.spd = ( (long) (pow( 1.08, (*(m_pRxBuffer+nOffset+1)-33) ) - 1) * (long) 6076 )/ (long) 5280;
        return CWND_LENGTH;
    }

    else                            // uncompressed
    {
        strxcpy( szTemp, m_pRxBuffer+nOffset, 3 );
        pData->wac.dir = atoi( szTemp );
        strxcpy( szTemp, m_pRxBuffer+nOffset+4, 3 );
        pData->wac.spd = ( atol( szTemp ) * (long) 6076 )/ (long) 5280;
        return UWND_LENGTH;
    }

}


//////////////////////////////////////////////////////////////////////////////
// WxnDecode - decode WXN specific data type on APRS port. Argument is
// structure to which data will be written. Returns TRUE success, FALSE on
// failure. Class variable m_nDecodeType is set to NO_TYPE, TYPE_A, TYPE_B,
// TYPE_C, TYPE_D or TYPE_E.
//
// Type a:
//  fips, city
//  alias
//  temp: now, hi, hi time, low, low time, hr rate, 24 hr rate
//  yday temps: hi, lo (no separator between this and previous group)
//  soil_temp
//
// Type b:
//  pca, pri, prh
//  plh, p24, pty, ptm, pta
//  ecc, eqc, exc, ehc, edc
//
// Type c:
//  wind: avg,dir, gust,dir, avghi,dir,time, avglo,dir,time, ghi,dir,time
//  haat
//
// Type d:
//  bar: now, hi, hi time, lo, lo time, hr rate, 24hr rate
//  humid:  now, hi, hi time, low, low time, hr rate, 24 hr rate
//  inside temp, humidity
//
// Type e:
//  rca, rha, rda, rmax, ralarm_count, ralarm_trip
//

BOOL Aprs::WxnDecode( noderx * pBlock )
{

    char * pS;
    char szString[128];
    char szTime1[16];
    char szTime2[16];
    char szTime3[16];

    m_nDecodeType = NO_TYPE;

    if( *(m_pRxBuffer+2) == 'a' )       // type a
    {
        ////////// fips, city
        // fips
        pS = strtok( m_szRxBuffer+3, "," );
        if( pS == NULL )
            return FALSE;
        TrimSpaces( pS );
        strxcpy( pBlock->Info.szFips, pS, FIPS_SIZE );

        // city
        pS = strtok( NULL, "/" );
        if( pS == NULL )
            return FALSE;
        TrimSpaces( pS );
        strxcpy( pBlock->Info.szCity, pS, CITY_SIZE );

        // alias
        pS = strtok( NULL, "/" );
        if( pS == NULL )
            return FALSE;
        TrimSpaces( pS );
        strxcpy( pBlock->Info.szAlias, pS, CALLSSID_SIZE );

        // temperature
        pS = strtok( NULL, "/" );
        if( pS == NULL)
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        SpaceDelimit( szString );
        if( sscanf( szString, "%i %i %s %i %s %i %i",
            &pBlock->Data.tcr,
            &pBlock->Data.thr.val, szTime1,
            &pBlock->Data.tlr.val, szTime2,
            &pBlock->Data.trh, &pBlock->Data.trd ) != 7 )
                return FALSE;
        pBlock->Data.thr.dtg = g_Time.MakeNodeDtg( szTime1 );
        pBlock->Data.tlr.dtg = g_Time.MakeNodeDtg( szTime2 );

        pS = strtok( NULL, "\r" );
        if( pS == NULL)
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        if( sscanf( szString, "%i,%i,%i", &pBlock->Data.tyh, &pBlock->Data.tyl, &pBlock->Data.tsd ) != 3 )
                return FALSE;


        m_nDecodeType = TYPE_A;

        return TRUE;

    }

    else if( *(m_pRxBuffer+2) == 'b' )    // type b
    {
        //////////  pca, pri, prh, plh, p24, pty, ptm, pta
        pS = strtok( m_szRxBuffer+3, "/" );
        if( pS == NULL )
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        if( sscanf( szString, "%i,%i,%i,%i,%i,%i,%i,%i",
            &pBlock->Data.pca,
            &pBlock->Data.pri,
            &pBlock->Data.prh,
            &pBlock->Data.plh,
            &pBlock->Data.p24,
            &pBlock->Data.pty,
            &pBlock->Data.ptm,
            &pBlock->Data.pta ) != 8 )
            return FALSE;

        //////////  emp 5, 15, 30, 60, day
        pS = strtok( NULL, "\r" );
        if( pS == NULL)
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        if( sscanf( szString, "%u,%u,%u,%u,%u",
            &pBlock->Data.ecc,
            &pBlock->Data.eqc,
            &pBlock->Data.exc,
            &pBlock->Data.ehc,
            &pBlock->Data.edc ) != 5 )
                return FALSE;

        m_nDecodeType = TYPE_B;

        return TRUE;
    }

    else if( *(m_pRxBuffer+2) == 'c' )    // type c
    {
        // wind
        pS = strtok( m_szRxBuffer+3, "/" );
        if( pS == NULL )
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );

        SpaceDelimit( szString );
        if( sscanf( szString, "%i %i %i %i %i %i %s %i %i %s %i %i %s",
            &pBlock->Data.wac.spd, &pBlock->Data.wac.dir,
            &pBlock->Data.wgc.spd, &pBlock->Data.wgc.dir,

            &pBlock->Data.wah.spd, &pBlock->Data.wah.dir, szTime1,
            &pBlock->Data.wal.spd, &pBlock->Data.wal.dir, szTime2,
            &pBlock->Data.wgh.spd, &pBlock->Data.wgh.dir, szTime3 ) != 13 )
                return FALSE;


        pBlock->Data.wah.dtg = g_Time.MakeNodeDtg( szTime1 );
        pBlock->Data.wal.dtg = g_Time.MakeNodeDtg( szTime2 );
        pBlock->Data.wgh.dtg = g_Time.MakeNodeDtg( szTime3 );

        ////////// HAAT
        pS = strtok( NULL, "\r" );
        if( pS == NULL)
            return FALSE;
        pBlock->Info.nHAAT = atoi( pS );

        m_nDecodeType = TYPE_C;

        return TRUE;
    }

    else if( *(m_pRxBuffer+2) == 'd' )    // type D
    {
        // barometer
        pS = strtok( m_szRxBuffer+3, "/" );
        if( pS == NULL )
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        SpaceDelimit( szString );
        if( sscanf( szString, "%i %i %s %i %s %i %i",
            &pBlock->Data.bcr,
            &pBlock->Data.bhr.val, szTime1,
            &pBlock->Data.blr.val, szTime2,
            &pBlock->Data.brh, &pBlock->Data.brd ) != 7 )
                return FALSE;

        pBlock->Data.bhr.dtg = g_Time.MakeNodeDtg( szTime1 );
        pBlock->Data.blr.dtg = g_Time.MakeNodeDtg( szTime2 );

        // humidity
        pS = strtok( NULL, "/" );
        if( pS == NULL)
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        SpaceDelimit( szString );
        if( sscanf( szString, "%i %i %s %i %s %i %i",
            &pBlock->Data.hcr,
            &pBlock->Data.hhr.val, szTime1,
            &pBlock->Data.hlr.val, szTime2,
            &pBlock->Data.hrh, &pBlock->Data.hrd ) != 7 )
                return FALSE;

        pBlock->Data.hhr.dtg = g_Time.MakeNodeDtg( szTime1 );
        pBlock->Data.hlr.dtg = g_Time.MakeNodeDtg( szTime2 );

        //////////  inside temp, humidity
        pS = strtok( NULL, "\r" );
        if( pS == NULL)
            return FALSE;
        strxcpy( szString, pS, STRING_SIZE );
        if( sscanf( szString, "%i,%i", &pBlock->Data.ist, &pBlock->Data.ish ) != 2 )
            return FALSE;

        m_nDecodeType = TYPE_D;

        return TRUE;
    }

    // background radiation count
    else if( *(m_pRxBuffer+2) == 'e' )    // type E
    {
        strxcpy( szString, &*(m_pRxBuffer+3), STRING_SIZE );
        if( sscanf( szString, "%i,%i,%i,%i,%i,%i",
            &pBlock->Data.rca,
            &pBlock->Data.rha,
            &pBlock->Data.rda,
            &pBlock->Data.rmax,
            &pBlock->Data.ralarm_count,
            &pBlock->Info.nRadAlarmTrip ) != 6 )
                return FALSE;

        m_nDecodeType = TYPE_E;

        return TRUE;
    }

    return FALSE;   // bad packet

}


//////////////////////////////////////////////////////////////////////////////
// DecodeType - returns last type decoded by WxnDecode(). If nothing decoded,
// returns NO_TYPE.
//

int Aprs::DecodeType( void )
{
    return m_nDecodeType;
}


//////////////////////////////////////////////////////////////////////////////
// WxnTx - transmit WXN specific data on APRS port. Argument determines if
// type A, b, c, or d is sent.
//
// Type a:
//  fips, city
//  alias
//  temp: now, hi, hi time, low, low time, hr rate, 24 hr rate
//  yday temps: hi, lo
//  soil_temp
//
// Type b:
//  pca, pri, prh
//  plh, p24, pty, ptm, pta
//  emp 5, 15, 30, 60, day
//
// Type c:
//  wind: avg,dir, gust,dir, avghi,dir,time, avglo,dir,time, ghi,dir,time
//  haat
//
// Type d:
//  bar: now, hi, hi time, lo, lo time, hr rate, 24hr rate
//  humid:  now, hi, hi time, low, low time, hr rate, 24 hr rate
//  inside temp, humidity
//
// Type e:
//  rca, rha, rda, rmax, rmax_count, rad_alarm_trip
//

void Aprs::WxnTx( int nType, noderx * pBlock )
{
    char szString[256];
    char szTime1[16];
    char szTime2[16];
    char szTime3[16];
    char szTime4[16];

    if( nType == TYPE_A )
    {
        g_Time.MakeNodeHrMin( pBlock->Data.thr.dtg, szTime1 );
        g_Time.MakeNodeHrMin( pBlock->Data.tlr.dtg, szTime2 );
        sprintf( szString,  "{%ca"

            "%s,"
            "%s/%s/"
            "%i,%i,%s,%i,%s,%i,%i/"
            "%i,%i,%i\r",

            WXN_USER_ID,

            pBlock->Info.szFips,
            pBlock->Info.szCity,
            pBlock->Info.szAlias,

            pBlock->Data.tcr,
            pBlock->Data.thr.val, szTime1,
            pBlock->Data.tlr.val, szTime2,

            pBlock->Data.trh,
            pBlock->Data.trd,

            pBlock->Data.tyh,
            pBlock->Data.tyl,
            pBlock->Data.tsd );
    }

    else if( nType == TYPE_B )
    {
        sprintf( szString,  "{%cb"
            "%i,%i,%i,%i,%i,%i,%i,%i/"
            "%u,%u,%u,%u,%u\r",

            WXN_USER_ID,

            pBlock->Data.pca,
            pBlock->Data.pri,
            pBlock->Data.prh,
            pBlock->Data.plh,
            pBlock->Data.p24,
            pBlock->Data.pty,
            pBlock->Data.ptm,
            pBlock->Data.pta,

            pBlock->Data.ecc,

            pBlock->Data.eqc,
            pBlock->Data.exc,
            pBlock->Data.ehc,

            pBlock->Data.edc );
    }

    else if( nType == TYPE_C )
    {
        g_Time.MakeNodeHrMin( pBlock->Data.wah.dtg, szTime1 );
        g_Time.MakeNodeHrMin( pBlock->Data.wal.dtg, szTime2 );
        g_Time.MakeNodeHrMin( pBlock->Data.wgh.dtg, szTime3 );

        sprintf( szString,  "{%cc"
            "%i,%i,%i,%i,%i,%i,%s,%i,%i,%s,%i,%i,%s/%i\r",

            WXN_USER_ID,

            pBlock->Data.wac.spd,
            pBlock->Data.wac.dir,
            pBlock->Data.wgc.spd,
            pBlock->Data.wgc.dir,
            pBlock->Data.wah.spd,
            pBlock->Data.wah.dir, szTime1,
            pBlock->Data.wal.spd,
            pBlock->Data.wal.dir, szTime2,
            pBlock->Data.wgh.spd,
            pBlock->Data.wgh.dir, szTime3,

            pBlock->Info.nHAAT );
    }

    else if( nType == TYPE_D )
    {
        g_Time.MakeNodeHrMin( pBlock->Data.bhr.dtg, szTime1 );
        g_Time.MakeNodeHrMin( pBlock->Data.blr.dtg, szTime2 );
        g_Time.MakeNodeHrMin( pBlock->Data.hhr.dtg, szTime3 );
        g_Time.MakeNodeHrMin( pBlock->Data.hlr.dtg, szTime4 );
        sprintf( szString,  "{%cd"
            "%i,%i,%s,%i,%s,%i,%i/"
            "%i,%i,%s,%i,%s,%i,%i/"
            "%i,%i\r",

            WXN_USER_ID,

            pBlock->Data.bcr,
            pBlock->Data.bhr.val, szTime1,
            pBlock->Data.blr.val, szTime2,
            pBlock->Data.brh,
            pBlock->Data.brd,

            pBlock->Data.hcr,
            pBlock->Data.hhr.val, szTime3,
            pBlock->Data.hlr.val, szTime4,
            pBlock->Data.hrh,

            pBlock->Data.hrd,

            pBlock->Data.ist,
            pBlock->Data.ish );
    }

    else if( nType == TYPE_E )

    {
        sprintf( szString,  "{%ce"
            "%i,%i,%i,%i,%i,%i\r",

            WXN_USER_ID,

            pBlock->Data.rca,

            pBlock->Data.rha,
            pBlock->Data.rda,
            pBlock->Data.rmax,
            pBlock->Data.ralarm_count,
            pBlock->Info.nRadAlarmTrip );
    }
    else        // unknown
        return;

    if( strlen(szString) > 256 )
    {
        g_log.PostMsg( APRMSG_TX_STRING_OVERFLOW, NULL );
        return;
    }

    BeaconWxn( szString );
}


//////////////////////////////////////////////////////////////////////////////
// Igate access routines

int Aprs::IGateCount( void )
{
    return m_nIGateCnt;
}


BOOL Aprs::IGateIndex( int nAction )
{
    switch( nAction )
    {
        case INDEX_RESET:
            m_nIGateIndex = 0;
            break;

        case INDEX_NEXT:
            if( (m_nIGateIndex+1) >= m_nIGateCnt )
                return FALSE;
            m_nIGateIndex++;
            break;

        case INDEX_PREV:
            if( m_nIGateIndex == 0 )
                return FALSE;
            m_nIGateIndex--;
            break;

        default:
            return FALSE;
    }

    return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// IGateDigi - return call in IGate digipeater string in the current index

void Aprs::IGateDigi( char * pCall )
{
    strcpy( pCall, m_sIDigi[m_nIGateIndex] );
}
