/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// dupe.cpp  -  Manage list of last n packets. Used to check for dupes in received packets.
//

#define _DUPE_CPP_

#include <string.h>

#include "dupe.h"

Dupe::Dupe()
{
    m_nCount = 0;
    m_pHead = NULL;
    m_pTail = NULL;
}

Dupe::~Dupe()
{
    dlist * pFree;
    dlist * pList;

    pList = m_pHead;

    while( pList )
    {
        pFree = pList;
        pList = pList->pNext;
        delete pFree;
    }
}

void Dupe::Add( char * szCallSsid, unsigned uCrc )
{
    dlist * pNew;

    if( m_nCount == MAX_LIST )
        RemoveTail();

    pNew = new dlist;
    if( !pNew )
        return;

    // set pointers
    pNew->pNext = m_pHead;
    pNew->pPrev = NULL;
    if( m_pHead )
        m_pHead->pPrev = pNew;
    else
        m_pTail = pNew;
    m_pHead = pNew;

    // set data
    pNew->uCrc = uCrc;
    strcpy( pNew->szCallSsid, szCallSsid );
    m_nCount++;
}


BOOL Dupe::InList( char * szCallSsid, unsigned uCrc )
{
    dlist * pList;

    pList = m_pHead;

    while( pList )
    {
        if( uCrc == pList->uCrc )
        {
            if( strcmp( szCallSsid, pList->szCallSsid ) == 0 )
                return TRUE;
        }
        pList = pList->pNext;
    }

    return FALSE;
}


void Dupe::RemoveTail( void )
{
    dlist * pFree;

    pFree = m_pTail;
    (m_pTail->pPrev)->pNext = NULL;
    m_pTail = m_pTail->pPrev;
    delete pFree;
    m_nCount--;

}

