/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// emp.cpp - handle EMP sensor data
//

#define _EMP_CPP_

#include <stdlib.h>
#include <libpq-fe.h>

// libwxn
#include <tz.h>

#include "onewire.h"
#include "wxdata.h"
#include "emp.h"


static unsigned long g_ulEmpLast;
static unsigned long g_ulEmpInitTotal;


unsigned long EmpInitTotal( void )
{
    return g_ulEmpInitTotal;
}


BOOL EmpInit( void )
{
    unsigned long  ulTemp;
    PGconn * pConn;
    PGresult * pResult;
    char szExec[STRING_SIZE];

    if( OwReadCounter( EMP_CNT, g_ulEmpLast ) )
        return FALSE;

    ulTemp = 0;
    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn )
    {
        if( PQstatus( pConn ) == CONNECTION_OK )
        {
            sprintf( szExec, "SELECT sum(ecount) FROM emp WHERE dtg_local>='%.4i-%.2i-%.2i 00:00'",
                g_Time.Year(), g_Time.Month(), g_Time.Day() );
            pResult = PQexec( pConn, szExec );
            if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
            {
                if( PQntuples( pResult ) )
                    ulTemp = (unsigned long) atoi( PQgetvalue( pResult, 0, 0 ) );
                PQclear( pResult );
            }
        PQfinish( pConn );
        }
    }

    g_ulEmpInitTotal = ulTemp;

    return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// EmpFix - used to determine current rad count which should be the difference
// between the current and the last. If counter has been reset, current accum
// becomes current reading. Writes accum to backend and returns current reading.
//

unsigned EmpFix( unsigned long ulVal )
{
    unsigned long ulCurrent;

    if( ulVal < g_ulEmpLast )
        ulCurrent = ulVal;
    else
        ulCurrent = ulVal - g_ulEmpLast;

    g_ulEmpLast = ulVal;

    return (unsigned) ulCurrent;
}


//////////////////////////////////////////////////////////////////////////////
// EmpSet - set 5, 15, 30, 60 minute totals plus daily total
//

void EmpSet( unsigned uCount )

{
    // 5 min
    g_Local.Data.ecc = uCount;

    // 15 min
    g_Local.Data.eqc = EmpGetTotal( 15 );

    // 30 min
    g_Local.Data.exc = EmpGetTotal( 30 );

    // 60 min
    g_Local.Data.ehc = EmpGetTotal( 60 );

    // day
    g_Local.Data.edc += uCount;
}


//////////////////////////////////////////////////////////////////////////////
// EmpGetTotal - get total EMP hits for nMinutes from backend
//

unsigned EmpGetTotal( int nMinutes )
{
    unsigned uTotal;
    PGconn * pConn;
    PGresult * pResult;
    char szExec[128];

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return 0;

    uTotal = 0;

    if( PQstatus( pConn ) == CONNECTION_OK )
    {
        sprintf( szExec, "SELECT sum(ecount) FROM emp WHERE dtg_local >= current_timestamp - interval '%i minutes'",
            nMinutes );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( PQntuples( pResult ) )
                uTotal = (unsigned) atoi( PQgetvalue( pResult, 0, 0 ) );
        }
        PQclear( pResult );
    }
    PQfinish( pConn );

    return uTotal;
}


//////////////////////////////////////////////////////////////////////////////
// EmpRead - gets current EMP hits from one-wire counter. If tare count is
// non-zero, also writes to backend.
//

void EmpRead( void )
{
    unsigned uCount;
    unsigned long ulAccum;
    PGconn * pConn;
    PGresult * pResult;
    char szExec[128];
    char szLocalTime[32];
    char szZuluTime[32];

    if( !OwReadCounter( EMP_CNT, ulAccum ) )
        return;

    uCount = EmpFix( ulAccum );

    if( uCount )
    {
        // save raw data
        pConn = PQconnectdb( "dbname=wxn" );
        if( pConn )
        {
            if( PQstatus( pConn ) == CONNECTION_OK )
            {
                g_Time.CurrentLocalTimestamp( szLocalTime );
                g_Time.CurrentZuluTimestamp( szZuluTime );
                sprintf( szExec, "INSERT INTO emp( dtg_local, dtg_zulu, ecount ) VALUES( '%s', '%s', %u )",
                    szLocalTime, szZuluTime, (unsigned) uCount );
                pResult = PQexec( pConn, szExec );
                PQclear( pResult );
            }
            PQfinish( pConn );
        }
    }

    EmpSet( uCount );
}


void EmpClear( void )
{
    g_Local.Data.edc = 0;
}

