/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// wind.cpp - data handler for wind
//

#define _WIND_CPP_

#include <stdlib.h>
#include <math.h>
#include <libpq-fe.h>

// libwxn
#include <datastruct.h>
#include <tz.h>

#include "wxdata.h"
#include "wind.h"


//////////////////////////////////////////////////////////////////////////////
// WindSet = sets wac,wgc,wgh,wah,wal
//

void WindSet( cwinds * pWind )
{
    PGconn * pConn;
    PGresult * pResult;
    char szExec[128];
    int nMin;

    //// WAC
    g_Local.Data.wac.spd = pWind->nSpeed;
    g_Local.Data.wac.dir = pWind->nDir;

    //// WAH
    if( g_Local.Data.wah.spd <= pWind->nSpeed )
    {
        g_Local.Data.wah.spd = pWind->nSpeed;
        g_Local.Data.wah.dir = pWind->nDir;
        g_Local.Data.wah.dtg = g_Time.Qth();
    }

    //// WAL
    if( g_Local.Data.wal.spd >= pWind->nSpeed )
    {
        g_Local.Data.wal.spd = pWind->nSpeed;
        g_Local.Data.wal.dir = pWind->nDir;
        g_Local.Data.wal.dtg = g_Time.Qth();
    }

    ////////////// wind gusts

    //// WGC
    g_Local.Data.wgc.spd = pWind->nGustSpeed;
    g_Local.Data.wgc.dir = pWind->nDir;

    //// WGH
    if( g_Local.Data.wgh.spd <= pWind->nGustSpeed )
    {
        g_Local.Data.wgh.spd = pWind->nGustSpeed;
        g_Local.Data.wgh.dir = pWind->nGustDir;
        g_Local.Data.wgh.dtg = g_Time.Qth();
    }

    // write to backend
    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn )
    {
        if( PQstatus( pConn ) == CONNECTION_OK )
        {
            // force minute to be multiple of 5
            nMin = (g_Time.Minute()/5) * 5;

            // save current reading
            sprintf( szExec, "UPDATE wind SET wgc_spd=%i, wgc_dir=%i, wac_spd=%i, wac_dir=%i WHERE hour=%i AND min=%i",
                g_Local.Data.wgc.spd, g_Local.Data.wgc.dir, g_Local.Data.wac.spd, g_Local.Data.wac.dir,
                g_Time.Hour(), nMin );
            pResult = PQexec( pConn, szExec );
            PQclear( pResult );
        }

        PQfinish( pConn );
    }
}


