/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// msgq.h  - manage the APRS message queue class
//

#ifndef _MSGQ_H
#define _MSGQ_H

#include <time.h>

// libwxn
#include <common.h>

#ifndef BOOL
    typedef int BOOL;
#endif

#define MAX_MSG_SIZE    70
#define MAX_MID_SIZE    8
#define MSG_CALL_SIZE   12

struct qlist
{
    char szFmCall[MSG_CALL_SIZE+1];
    unsigned uCrc;
    char szResp[MAX_MSG_SIZE+1];
    char szMID[MAX_MID_SIZE+1];
    int nRetries;
    BOOL bDelete;
    time_t tTimeout;
    qlist * pNext;
    qlist * pPrev;
};


class MsgQ
{
    public:

        MsgQ();
        ~MsgQ();

        void Init( int nRetries, time_t tTimeout, time_t tDelay );
        void Add( char * szFmCall, char * szResp, int nMID, unsigned uCrc );
        BOOL InList( char * szCallSsid, unsigned uCrc );
        qlist * ChkAck( void );
        BOOL Remove( char * szMID );
        void Remove( qlist * pNode );
        void RemoveOld( void );

    private:

        int m_nCount;

        int m_nRetries;
        time_t m_tTimeout;
        time_t m_tDelay;

        qlist * m_pHead;
        qlist * m_pTail;
};

#endif  // _MSGQ_H
